@@ -3,7 +3,8 @@ import time
 import traceback
 from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
-from typing import Any, Iterator, List
+from queue import SimpleQueue
+from typing import TYPE_CHECKING, Any, Iterator, List
 
 from logzero import logger
 
@@ -16,6 +17,9 @@ from chaoslib.provider.process import run_process_activity, validate_process_act
 from chaoslib.provider.python import run_python_activity, validate_python_activity
 from chaoslib.types import Activity, Configuration, Dry, Experiment, Run, Secrets
 
+if TYPE_CHECKING:
+    from chaoslib.run import EventHandlerRegistry
+
 __all__ = [
     "ensure_activity_is_valid",
     "get_all_activities_in_experiment",
@@ -116,6 +120,8 @@ def run_activities(
     secrets: Secrets,
     pool: ThreadPoolExecutor,
     dry: Dry = None,
+    event_registry: "EventHandlerRegistry" = None,
+    runs: SimpleQueue = None,
 ) -> Iterator[Run]:
     """
     Internal generator that iterates over all activities and execute them.
@@ -137,6 +143,8 @@ def run_activities(
                 configuration=configuration,
                 secrets=secrets,
                 dry=dry,
+                event_registry=event_registry,
+                runs=runs,
             )
         else:
             yield execute_activity(
@@ -145,6 +153,8 @@ def run_activities(
                 configuration=configuration,
                 secrets=secrets,
                 dry=dry,
+                event_registry=event_registry,
+                runs=runs,
             )
 
 
@@ -157,6 +167,8 @@ def execute_activity(
     configuration: Configuration,
     secrets: Secrets,
     dry: Dry,
+    event_registry: "EventHandlerRegistry" = None,
+    runs: SimpleQueue = None,
 ) -> Run:
     """
     Low-level wrapper around the actual activity provider call to collect
@@ -206,12 +218,15 @@ def execute_activity(
             )
 
         start = datetime.utcnow()
-
-        run = {"activity": activity.copy(), "output": None}
+        run = {"activity": activity.copy(), "output": None, "start": start.isoformat()}
+        if runs:
+            runs.put(run)
 
         result = None
         interrupted = False
         try:
+            if event_registry:
+                event_registry.start_activity(activity)
             # pause when one of the dry flags are set
             if not is_dry:
                 result = run_activity(activity, configuration, secrets)
@@ -230,10 +245,12 @@ def execute_activity(
         finally:
             # capture the end time before we pause
             end = datetime.utcnow()
-            run["start"] = start.isoformat()
             run["end"] = end.isoformat()
             run["duration"] = (end - start).total_seconds()
 
+            if event_registry:
+                event_registry.activity_completed(activity, run)
+
             pause_after = pauses.get("after")
             if pause_after and not interrupted:
                 logger.info(f"Pausing after activity for {pause_after}s...")
