@@ -1,14 +1,13 @@
-# -*- coding: utf-8 -*-
 # Builds an in-memory cache of all declared activities so they can be
 # referenced from other places in the experiment
-from functools import wraps
 import inspect
-from typing import Any, List, Dict, Union
+from functools import wraps
+from typing import Any, Dict, List, Union
 
 from logzero import logger
 
-from chaoslib.types import Activity, Experiment, Settings, Schedule, Strategy
-
+import chaoslib
+from chaoslib.types import Activity, Experiment, Schedule, Settings, Strategy
 
 __all__ = ["cache_activities", "clear_cache", "lookup_activity", "with_cache"]
 
@@ -24,15 +23,16 @@ def cache_activities(experiment: Experiment) -> List[Activity]:
     """
     logger.debug("Building activity cache...")
 
-    lot = experiment.get("method", []) + \
-        experiment.get("steady-state-hypothesis", {}).get("probes", [])
+    lot = experiment.get("method", []) + experiment.get(
+        "steady-state-hypothesis", {}
+    ).get("probes", [])
 
     for activity in lot:
         name = activity.get("name")
         if name:
             _cache[name] = activity
 
-    logger.debug("Cached {d} activities".format(d=len(_cache)))
+    logger.debug(f"Cached {len(_cache)} activities")
 
 
 def clear_cache():
@@ -48,20 +48,22 @@ def with_cache(f):
     Ensure the activities cache is populated before calling the wrapped
     function.
     """
+
     @wraps(f)
-    def wrapped(experiment: Experiment, settings: Settings = None,
-                experiment_vars: Dict[str, Any] = None,
-                strategy: Strategy = Strategy.DEFAULT,
-                schedule: Schedule = None,
-                event_handlers: List['RunEventHandler'] = None):  #noqa: E0602
+    def wrapped(
+        experiment: Experiment,
+        settings: Settings = None,
+        experiment_vars: Dict[str, Any] = None,
+        strategy: Strategy = Strategy.DEFAULT,
+        schedule: Schedule = None,
+        event_handlers: List[chaoslib.run.RunEventHandler] = None,
+    ):
         try:
             if experiment:
                 cache_activities(experiment)
 
             sig = inspect.signature(f)
-            arguments = {
-                "experiment": experiment
-            }
+            arguments = {"experiment": experiment}
 
             if "settings" in sig.parameters:
                 arguments["settings"] = settings
@@ -76,6 +78,7 @@ def with_cache(f):
             return f(**arguments)
         finally:
             clear_cache()
+
     return wrapped
 
 
@@ -85,5 +88,5 @@ def lookup_activity(ref: str) -> Union[Activity, None]:
     """
     activity = _cache.get(ref)
     if not activity:
-        logger.debug("cache miss for '{r}'".format(r=ref))
+        logger.debug(f"cache miss for '{ref}'")
     return activity
