@@ -1,4 +1,6 @@
+from abc import ABCMeta
 from concurrent.futures import Future, ThreadPoolExecutor, TimeoutError
+from queue import Empty, SimpleQueue
 
 try:
     import ctypes
@@ -36,6 +38,7 @@ from chaoslib.rollback import run_rollbacks
 from chaoslib.secret import load_secrets
 from chaoslib.settings import get_loaded_settings
 from chaoslib.types import (
+    Activity,
     Configuration,
     Dry,
     Experiment,
@@ -50,7 +53,7 @@ from chaoslib.types import (
 __all__ = ["Runner", "RunEventHandler"]
 
 
-class RunEventHandler:
+class RunEventHandler(metaclass=ABCMeta):
     """
     Base class to react to certain, or all, events during an execution.
 
@@ -59,7 +62,7 @@ class RunEventHandler:
     """
 
     def started(self, experiment: Experiment, journal: Journal) -> None:
-        logger.debug("Experiment execution started")
+        pass
 
     def running(
         self,
@@ -70,61 +73,67 @@ class RunEventHandler:
         schedule: Schedule,
         settings: Settings,
     ) -> None:
-        logger.debug("Experiment execution running")
+        pass
 
     def finish(self, journal: Journal) -> None:
-        logger.debug("Experiment execution finished")
+        pass
 
     def interrupted(self, experiment: Experiment, journal: Journal) -> None:
-        logger.debug("Experiment execution was interrupted by a control")
+        pass
 
     def signal_exit(self) -> None:
-        logger.debug("Experiment execution caught a system signal")
+        pass
 
     def start_continuous_hypothesis(self, frequency: int) -> None:
-        logger.debug("Steady state will run continuously now")
+        pass
 
     def continuous_hypothesis_iteration(self, iteration_index: int, state: Any) -> None:
-        logger.debug(f"Steady state iteration {iteration_index}")
+        pass
 
     def continuous_hypothesis_completed(
         self, experiment: Experiment, journal: Journal, exception: Exception = None
     ) -> None:
-        logger.debug("Continuous steady state is now complete")
+        pass
 
     def start_hypothesis_before(self, experiment: Experiment) -> None:
-        logger.debug("Steady state hypothesis is now running before method")
+        pass
 
     def hypothesis_before_completed(
         self, experiment: Experiment, state: Dict[str, Any], journal: Journal
     ) -> None:
-        logger.debug("Steady state hypothesis before method completed")
+        pass
 
     def start_hypothesis_after(self, experiment: Experiment) -> None:
-        logger.debug("Steady state hypothesis is now running after method")
+        pass
 
     def hypothesis_after_completed(
         self, experiment: Experiment, state: Dict[str, Any], journal: Journal
     ) -> None:
-        logger.debug("Steady state hypothesis after method completed")
+        pass
 
     def start_method(self, experiment: Experiment) -> None:
-        logger.debug("Method has now started")
+        pass
 
     def method_completed(self, experiment: Experiment, state: Any) -> None:
-        logger.debug("Method has now completed")
+        pass
 
     def start_rollbacks(self, experiment: Experiment) -> None:
-        logger.debug("Rollbacks have now started")
+        pass
 
     def rollbacks_completed(self, experiment: Experiment, journal: Journal) -> None:
-        logger.debug("Rollbacks have now completed")
+        pass
 
     def start_cooldown(self, duration: int) -> None:
-        logger.debug("Cooldown period has now started")
+        pass
 
     def cooldown_completed(self) -> None:
-        logger.debug("Cooldown period has now completed")
+        pass
+
+    def start_activity(self, activity: Activity) -> None:
+        pass
+
+    def activity_completed(self, activity: Activity, run: Run) -> None:
+        pass
 
 
 class EventHandlerRegistry:
@@ -276,6 +285,20 @@ class EventHandlerRegistry:
             except Exception:
                 logger.debug(f"Handler {h.__class__.__name__} failed", exc_info=True)
 
+    def start_activity(self, activity: Activity) -> None:
+        for h in self.handlers:
+            try:
+                h.start_activity(activity)
+            except Exception:
+                logger.debug(f"Handler {h.__class__.__name__} failed", exc_info=True)
+
+    def activity_completed(self, activity: Activity, run: Run) -> None:
+        for h in self.handlers:
+            try:
+                h.activity_completed(activity, run)
+            except Exception:
+                logger.debug(f"Handler {h.__class__.__name__} failed", exc_info=True)
+
 
 class Runner:
     def __init__(self, strategy: Strategy, schedule: Schedule = None):
@@ -536,7 +559,9 @@ def run_gate_hypothesis(
     """
     logger.debug("Running steady-state hypothesis before the method")
     event_registry.start_hypothesis_before(experiment)
-    state = run_steady_state_hypothesis(experiment, configuration, secrets, dry=dry)
+    state = run_steady_state_hypothesis(
+        experiment, configuration, secrets, dry=dry, event_registry=event_registry
+    )
     journal["steady_states"]["before"] = state
     event_registry.hypothesis_before_completed(experiment, state, journal)
     if state is not None and not state["steady_state_met"]:
@@ -566,7 +591,9 @@ def run_deviation_validation_hypothesis(
     """
     logger.debug("Running steady-state hypothesis after the method")
     event_registry.start_hypothesis_after(experiment)
-    state = run_steady_state_hypothesis(experiment, configuration, secrets, dry=dry)
+    state = run_steady_state_hypothesis(
+        experiment, configuration, secrets, dry=dry, event_registry=event_registry
+    )
     journal["steady_states"]["after"] = state
     event_registry.hypothesis_after_completed(experiment, state, journal)
     if state is not None and not state["steady_state_met"]:
@@ -638,9 +665,18 @@ def run_method(
     logger.info("Playing your experiment's method now...")
     event_registry.start_method(experiment)
     try:
-        state = apply_activities(
-            experiment, configuration, secrets, activity_pool, journal, dry
+        runs = SimpleQueue()
+        apply_activities(
+            experiment,
+            configuration,
+            secrets,
+            activity_pool,
+            journal,
+            dry,
+            event_registry,
+            runs=runs,
         )
+        state = journal["run"]
         event_registry.method_completed(experiment, state)
         return state
     except InterruptExecution:
@@ -696,7 +732,7 @@ def run_rollback(
         event_registry.start_rollbacks(experiment)
         try:
             journal["rollbacks"] = apply_rollbacks(
-                experiment, configuration, secrets, rollback_pool, dry
+                experiment, configuration, secrets, rollback_pool, dry, event_registry
             )
         except InterruptExecution as i:
             journal["status"] = "interrupted"
@@ -802,7 +838,9 @@ def run_hypothesis_continuously(
         if journal["status"] in ["failed", "interrupted", "aborted"]:
             break
 
-        state = run_steady_state_hypothesis(experiment, configuration, secrets, dry=dry)
+        state = run_steady_state_hypothesis(
+            experiment, configuration, secrets, dry=dry, event_registry=event_registry
+        )
         journal["steady_states"]["during"].append(state)
         event_registry.continuous_hypothesis_iteration(iteration, state)
 
@@ -840,6 +878,8 @@ def apply_activities(
     pool: ThreadPoolExecutor,
     journal: Journal,
     dry: Dry,
+    event_registry: EventHandlerRegistry,
+    runs: SimpleQueue,
 ) -> List[Run]:
     with controls(
         level="method",
@@ -849,13 +889,21 @@ def apply_activities(
         secrets=secrets,
     ) as control:
         result = []
-        runs = []
-        method = experiment.get("method", [])
+        futures = []
         wait_for_background_activities = True
 
         try:
-            for run in run_activities(experiment, configuration, secrets, pool, dry):
-                runs.append(run)
+            for run in run_activities(
+                experiment,
+                configuration,
+                secrets,
+                pool,
+                dry,
+                event_registry,
+                runs,
+            ):
+                if isinstance(run, Future):
+                    futures.append(run)
                 if journal["status"] in ["aborted", "failed", "interrupted"]:
                     break
         except SystemExit as x:
@@ -865,7 +913,15 @@ def apply_activities(
             wait_for_background_activities = x.code != 30  # see exit.py
             raise
         finally:
-            background_activity_timeout = None
+            while True:
+                try:
+                    result.append(runs.get_nowait())
+                except Empty:
+                    break
+
+            journal["run"] = result
+
+            control.with_state(result)
 
             if wait_for_background_activities and pool:
                 logger.debug("Waiting for background activities to complete")
@@ -876,40 +932,15 @@ def apply_activities(
                     "Do not wait for the background activities to finish "
                     "as per signal"
                 )
-                background_activity_timeout = 0.2
-                pool.shutdown(wait=False)
 
-            for index, run in enumerate(runs):
-                if not run:
-                    continue
-
-                if isinstance(run, dict):
-                    result.append(run)
-                else:
+                for f in futures:
                     try:
-                        # background activities
-                        result.append(run.result(timeout=background_activity_timeout))
+                        if f.running():
+                            f.result(timeout=0.2)
                     except TimeoutError:
-                        # we want an entry for the background activity in our
-                        # results anyway, we won't have anything meaningful
-                        # to say about it
-                        result.append(
-                            {
-                                "activity": method[index],
-                                "status": "failed",
-                                "output": None,
-                                "duration": None,
-                                "start": None,
-                                "end": None,
-                                "exception": None,
-                            }
-                        )
+                        pass
 
-            # now let's ensure the journal has all activities in their correct
-            # order (background ones included)
-            journal["run"] = result
-
-            control.with_state(result)
+                pool.shutdown(wait=False)
 
     return result
 
@@ -920,6 +951,7 @@ def apply_rollbacks(
     secrets: Secrets,
     pool: ThreadPoolExecutor,
     dry: Dry,
+    event_registry: EventHandlerRegistry,
 ) -> List[Run]:
     logger.info("Let's rollback...")
     with controls(
@@ -929,7 +961,9 @@ def apply_rollbacks(
         configuration=configuration,
         secrets=secrets,
     ) as control:
-        rollbacks = list(run_rollbacks(experiment, configuration, secrets, pool, dry))
+        rollbacks = list(
+            run_rollbacks(experiment, configuration, secrets, pool, dry, event_registry)
+        )
 
         if pool:
             logger.debug("Waiting for background rollbacks to complete...")
