@@ -1,13 +1,33 @@
-# -*- coding: utf-8 -*-
 import enum
-from typing import Any, Dict, List, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
-__all__ = ["MicroservicesStatus", "Probe", "Action", "Experiment", "Layer",
-           "TargetLayers", "Activity", "Journal", "Run", "Secrets", "Step",
-           "Configuration", "Discovery", "DiscoveredActivities", "Extension",
-           "DiscoveredSystemInfo", "Settings", "EventPayload", "Tolerance",
-           "Hypothesis", "Control", "Strategy", "Schedule",
-           "ConfigVars", "SecretVars"]
+__all__ = [
+    "MicroservicesStatus",
+    "Probe",
+    "Action",
+    "Experiment",
+    "Layer",
+    "TargetLayers",
+    "Activity",
+    "Journal",
+    "Run",
+    "Secrets",
+    "Step",
+    "Configuration",
+    "Discovery",
+    "DiscoveredActivities",
+    "Extension",
+    "DiscoveredSystemInfo",
+    "Settings",
+    "EventPayload",
+    "Tolerance",
+    "Hypothesis",
+    "Control",
+    "Strategy",
+    "Schedule",
+    "ConfigVars",
+    "SecretVars",
+]
 
 
 Action = Dict[str, Any]
@@ -49,10 +69,10 @@ class Strategy(enum.Enum):
     AFTER_METHOD = "after-method-only"
     DURING_METHOD = "during-method-only"
     DEFAULT = "default"
-    CONTINOUS = "continous"
+    CONTINUOUS = "continuous"
 
     @staticmethod
-    def from_string(value: str) -> 'Strategy':
+    def from_string(value: str) -> "Strategy":
         if value == "default":
             return Strategy.DEFAULT
         elif value == "before-method-only":
@@ -61,17 +81,41 @@ class Strategy(enum.Enum):
             return Strategy.AFTER_METHOD
         elif value == "during-method-only":
             return Strategy.DURING_METHOD
-        elif value == "continously":
-            return Strategy.CONTINOUS
-        elif value == "continously-fail-fast":
-            return Strategy.CONTINOUS
+        elif value == "continuously":
+            return Strategy.CONTINUOUS
 
         raise ValueError("Unknown strategy")
 
 
+class Dry(enum.Enum):
+    PROBES = "probes"
+    ACTIONS = "actions"
+    ACTIVITIES = "activities"
+    PAUSE = "pause"
+
+    @staticmethod
+    def from_string(value: str) -> Optional["Dry"]:
+        if value == "probes":
+            return Dry.PROBES
+        elif value == "actions":
+            return Dry.ACTIONS
+        elif value == "activities":
+            return Dry.ACTIVITIES
+        elif value == "pause":
+            return Dry.PAUSE
+        elif not value:
+            return None
+
+        raise ValueError("Unknown dry")
+
+
 class Schedule:
-    def __init__(self, continous_hypothesis_frequency: float = 1.0,
-                 fail_fast: bool = False, fail_fast_ratio: float = 0):
-        self.continous_hypothesis_frequency = continous_hypothesis_frequency
+    def __init__(
+        self,
+        continuous_hypothesis_frequency: float = 1.0,
+        fail_fast: bool = False,
+        fail_fast_ratio: float = 0,
+    ):
+        self.continuous_hypothesis_frequency = continuous_hypothesis_frequency
         self.fail_fast = fail_fast
         self.fail_fast_ratio = fail_fast_ratio
